/**
 * 
 */
package org.mycompany.connector;

import java.util.ArrayList;
import java.util.List;

import org.bonitasoft.engine.api.IdentityAPI;
import org.bonitasoft.engine.connector.ConnectorValidationException;
import org.bonitasoft.engine.filter.UserFilterException;
import org.bonitasoft.engine.identity.UserCriterion;
import org.bonitasoft.engine.identity.UserMembershipCriterion;
import org.bonitasoft.engine.identity.UserMembership;
import org.bonitasoft.engine.identity.User;
import org.bonitasoft.engine.identity.Group;

/**
 *The actor filter execution will follow the steps
 * 1 - setInputParameters() --> the actor filter receives input parameters values
 * 2 - validateInputParameters() --> the actor filter can validate input parameters values
 * 3 - filter(final String actorName) --> execute the user filter
 * 4 - shouldAutoAssignTaskIfSingleResult() --> auto-assign the task if filter returns a single result
 */
public class InitiatorVorgesetzterDynamischImpl extends
		AbstractInitiatorVorgesetzterDynamischImpl {

	@Override
	public void validateInputParameters() throws ConnectorValidationException {
		//TODO validate input parameters here 
	
	}

	@Override
	public List<Long> filter(final String actorName) throws UserFilterException {
		
		List<Long> userList = new ArrayList<Long>();
		IdentityAPI ia = getAPIAccessor().getIdentityAPI();

		try{
			// find memberships of initiator
			Long initiatorID = getAPIAccessor().getProcessAPI().getProcessInstance(getExecutionContext().getProcessInstanceId()).getStartedBy();
			int noOfMemberships = (int)(ia.getNumberOfUserMemberships(initiatorID));
			List<UserMembership> membershipList = ia.getUserMemberships(initiatorID, 0, noOfMemberships, UserMembershipCriterion.GROUP_NAME_ASC);
			// traverse Groups of initiator's memberships
			for(UserMembership iMembership:membershipList){
				Long iGroupID = iMembership.getGroupId();
				userList.addAll(getManagersInGroup(iGroupID));
			}
		} catch(Exception e){
		}
		
		
		//TODO execute the user filter here
		//The method must return a list of actor id's
		//you can use getApiAccessor() and getExecutionContext()
		return userList;
	
	}
	
	/**
	 * Search the group and its parent groups for users who have the role "Leiter" in 
	 * the respective group. 
	 * @param groupID the ID of the group to start searching
	 * @return the list of found users' IDs
	 */
	private List<Long> getManagersInGroup(Long groupID){
		List<Long> managers = new ArrayList<Long>();
		
		IdentityAPI ia = getAPIAccessor().getIdentityAPI();
		
		try{
		
			int noOfUsers = (int)(ia.getNumberOfUsersInGroup(groupID));
			
			// traverse users in group	
			List<User> users = ia.getUsersInGroup(groupID, 0, noOfUsers, UserCriterion.LAST_NAME_ASC);
			for(User iUser:users){
				Long iUserID = iUser.getId();
				
				
	
				
				
				int noOfUsersMemberships = (int)(ia.getNumberOfUserMemberships(iUserID));
				// traverse users' memberships
				List<UserMembership> usersMemberships = ia.getUserMemberships(iUserID, 0, noOfUsersMemberships, UserMembershipCriterion.GROUP_NAME_ASC);
				for(UserMembership iUsersMembership:usersMemberships){
					if ((iUsersMembership.getGroupId() == groupID) &&
					    (iUsersMembership.getRoleName().equals("Leiter"))){
						managers.add(iUserID);
					}
				}
			}
			
			try{
				// call method recursively for parent group
				Long parentGroupID = ia.getGroupByPath(ia.getGroup(groupID).getParentPath()).getId();
				managers.addAll(getManagersInGroup(parentGroupID));
			} catch (Exception e){
			}
			
		} catch(Exception e){
		}

		return managers;
	}
	
	
	

	@Override
	public boolean shouldAutoAssignTaskIfSingleResult() {
		// If this method returns true, the step will be assigned to 
		//the user if there is only one result returned by the filter method
		return super.shouldAutoAssignTaskIfSingleResult();
	
	}

}
